from __future__ import annotations

from textual.app import ComposeResult, events
from textual.screen import ModalScreen, Screen
from textual.widgets import Static

from hoptex.configs import JUMPS, HoptexBindingConfig, HoptexWidgetsFiltersConfig
from hoptex.utils import get_focusable


class HopScreen(ModalScreen):
    """hoptex Screen that provides point to jump"""

    def __init__(
        self, bindings: HoptexBindingConfig, filters_lists: HoptexWidgetsFiltersConfig, label: type[Static]
    ) -> None:
        super().__init__()
        self._bindings.bind(*(bindings.quit, "return", "Return"))
        self.filters_lists = filters_lists
        self.label = label

    def set_parent_screen(self, screen: Screen, click: bool = False):
        self.data_screen = screen
        self.should_just_click = click

    def compose(self) -> ComposeResult:
        self.to_choose = dict()

        for idx, i in enumerate(tuple(get_focusable(self.data_screen, self.filters_lists))):
            bar = self.label(JUMPS[-1 - idx])
            self.mount(bar)
            bar.styles.layer = f"new_layer_{idx}"
            bar.styles.offset = (i.region.x, i.region.y)
            self.to_choose[JUMPS[-1 - idx]] = i

        yield from ()

    def on_key(self, event: events.Key) -> None:
        if event.key in self.to_choose:
            self.dismiss(self.to_choose[event.key])

    def action_return(self) -> None:
        self.app.pop_screen()



from __future__ import annotations

from collections.abc import Iterator

from textual.widget import Widget, events

from hoptex.configs import HoptexWidgetsFiltersConfig

DUMMY_MOUSE_EVENT = events.Click(0, 0, 0, 0, 0, False, False, False, False)


def focusable_children(widget: Widget) -> list[Widget]:
    focusable = [child for child in widget._nodes if child.display and child.visible]
    return focusable


def get_focusable(widget: Widget, filter_lists: HoptexWidgetsFiltersConfig):
    widgets: list[Widget] = []
    add_widget = widgets.append
    stack: list[Iterator[Widget]] = [iter(focusable_children(widget))]
    pop = stack.pop
    push = stack.append

    while stack:
        node = next(stack[-1], None)
        if node is None:
            pop()
        else:
            if type(node) in filter_lists.block_list:
                continue
            if node.is_container and node.can_focus_children:
                push(iter(node.focusable_children))
            if filter_lists.include_focusable and node.focusable:
                add_widget(node)
            elif type(node) in filter_lists.allow_list:
                add_widget(node)

    return widgets



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Dict, Union

from textual.widgets import Static

if TYPE_CHECKING:
    from typing_extensions import TypeAlias


BINDING_QUIT_HOP = "escape"
BINDING_START_HOP = "space"
BINDING_PRESS_HOP = "ctrl+o"

BINDING_UNFOCUS_HOP = "escape"

JUMPS = "asdghklqwertyuiopzxcvbnmfj"

ConfType: TypeAlias = Dict[str, Union[str, bool]]


@dataclass
class HoptexBindingConfig:
    focus: str = BINDING_START_HOP
    press: str = BINDING_PRESS_HOP
    quit: str = BINDING_QUIT_HOP
    unfocus: str = BINDING_UNFOCUS_HOP

    focus_conf: ConfType = field(default_factory=lambda: {"description": "Hop Focus"})
    press_conf: ConfType = field(default_factory=lambda: {"description": "Hop Press"})
    quit_conf: ConfType = field(default_factory=lambda: {"description": "Hop Quit", "show": False})
    unfocus_conf: ConfType = field(default_factory=lambda: {"description": "Hop Unfocus", "show": False})


@dataclass
class HoptexWidgetsFiltersConfig:
    include_focusable: bool = True
    allow_list: list[type] = field(default_factory=lambda: [])
    block_list: list[type] = field(default_factory=lambda: [])


class HopLabel(Static):
    DEFAULT_CSS = """
    HopLabel {
        width: 1;
        height: 1;
        color: yellow;
        text-style: bold;
        background: black;
    }
    """



from __future__ import annotations

import inspect

from textual.app import App
from textual.widget import Widget
from textual.widgets import Static

from hoptex.configs import HopLabel, HoptexBindingConfig, HoptexWidgetsFiltersConfig
from hoptex.hop_screen import HopScreen
from hoptex.utils import DUMMY_MOUSE_EVENT


async def _action_hoptex_screen_choose(self: App, focus_and_press: bool = False) -> None:
    bindings = getattr(self, "_hoptex_bindings", None)
    filter_lists = getattr(self, "_hoptex_filter_lists", None)
    label = getattr(self, "_hoptex_label", HopLabel)

    if not bindings or not filter_lists:
        raise AttributeError("Missing bindings or filter lists")

    screen = HopScreen(bindings, filter_lists, label)

    screen.set_parent_screen(self.screen)

    async def _perform_action(widget: Widget):
        for function, attrs, kwargs in [("_on_click", [DUMMY_MOUSE_EVENT], {}), ("on_click", [], {})]:
            if not hasattr(widget, function):
                continue

            if inspect.iscoroutinefunction(getattr(widget, function)):
                await getattr(widget, function)(*attrs, **kwargs)
            else:
                getattr(widget, function)(*attrs, **kwargs)

    async def change_focus(widget: Widget):
        self.set_focus(widget)
        if not focus_and_press:
            return

        await _perform_action(widget)

    self.push_screen(screen, change_focus)


async def _action_hoptex_unfocus(self: App) -> None:
    self.screen.set_focus(None)


def hoptex(
    cls=None,
    *,
    bindings: HoptexBindingConfig = HoptexBindingConfig(),
    widgets_filters: HoptexWidgetsFiltersConfig = HoptexWidgetsFiltersConfig(),
    label: type[Static] = HopLabel,
):
    def _hoptex_decorator(cls):
        original_init = cls.__init__

        def hoptex_init(self, *args, **kwargs):
            original_init(self, *args, **kwargs)
            self._bindings.bind(*(bindings.focus, "hoptex_screen_choose"), **bindings.focus_conf)
            self._bindings.bind(*(bindings.press, "hoptex_screen_choose(True)"), **bindings.press_conf)
            self._bindings.bind(*(bindings.quit, "hoptex_unfocus"), **bindings.quit_conf)

        cls.__init__ = hoptex_init

        setattr(cls, "_action_hoptex_screen_choose", _action_hoptex_screen_choose)
        setattr(cls, "_action_hoptex_unfocus", _action_hoptex_unfocus)

        setattr(cls, "_hoptex_bindings", bindings)
        setattr(cls, "_hoptex_filter_lists", widgets_filters)

        setattr(cls, "_hoptex_label", label)

        return cls

    if cls is None:
        return _hoptex_decorator

    return _hoptex_decorator(cls)
